==> rest-api-1.1.0-sources.jar: software/rsquared/restapi/RequestExecutor.java <==
package software.rsquared.restapi;

import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import software.rsquared.restapi.exceptions.RequestException;
import software.rsquared.restapi.listeners.RequestListener;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.Callable;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.RejectedExecutionHandler;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * Wrapper for the {@link ThreadPoolExecutor} that allows to send signal to the {@link RequestListener} if execution finished and wraps exceptions to the {@link RequestException}
 *
 * @author Rafał Zajfert
 */
@SuppressWarnings("unused")
class RequestExecutor extends ThreadPoolExecutor {

    //region Constructors
    /**
     * Creates a new {@code RequestExecutor} default initial
     * parameters, thread factory and rejected execution handler.
     */
    RequestExecutor() {
        this(1, 0L);
    }

    /**
     * Creates a new {@code RequestExecutor} with the given initial
     * parameters and default thread factory and rejected execution handler.
     *
     * @param poolSize      the number of threads to keep in the pool, even
     *                      if they are idle, unless {@code allowCoreThreadTimeOut} is set
     * @param aliveDuration when the number of threads is greater than
     *                      the core, this is the maximum time (in milliseconds) that excess idle threads
     *                      will wait for new tasks before terminating.
     * @throws IllegalArgumentException if one of the following holds:<br>
     *                                  {@code corePoolSize < 0}<br>
     *                                  {@code keepAliveTime < 0}<br>
     */
    RequestExecutor(int poolSize, long aliveDuration) {
        super(poolSize, poolSize, aliveDuration, TimeUnit.MILLISECONDS, new LinkedBlockingQueue<Runnable>());
    }

    /**
     * Creates a new {@code RequestExecutor} with the given initial
     * parameters and default thread factory and rejected execution handler.
     *
     * @param corePoolSize    the number of threads to keep in the pool, even
     *                        if they are idle, unless {@code allowCoreThreadTimeOut} is set
     * @param maximumPoolSize the maximum number of threads to allow in the
     *                        pool
     * @param keepAliveTime   when the number of threads is greater than
     *                        the core, this is the maximum time that excess idle threads
     *                        will wait for new tasks before terminating.
     * @param unit            the time unit for the {@code keepAliveTime} argument
     * @param workQueue       the queue to use for holding tasks before they are
     *                        executed.  This queue will hold only the {@code Runnable}
     *                        tasks submitted by the {@code execute} method.
     * @throws IllegalArgumentException if one of the following holds:<br>
     *                                  {@code corePoolSize < 0}<br>
     *                                  {@code keepAliveTime < 0}<br>
     *                                  {@code maximumPoolSize <= 0}<br>
     *                                  {@code maximumPoolSize < corePoolSize}
     * @throws NullPointerException     if {@code workQueue} is null
     */
    public RequestExecutor(int corePoolSize, int maximumPoolSize, long keepAliveTime, TimeUnit unit, BlockingQueue<Runnable> workQueue) {
        super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue);
    }

    /**
     * Creates a new {@code RequestExecutor} with the given initial
     * parameters and default rejected execution handler.
     *
     * @param corePoolSize    the number of threads to keep in the pool, even
     *                        if they are idle, unless {@code allowCoreThreadTimeOut} is set
     * @param maximumPoolSize the maximum number of threads to allow in the
     *                        pool
     * @param keepAliveTime   when the number of threads is greater than
     *                        the core, this is the maximum time that excess idle threads
     *                        will wait for new tasks before terminating.
     * @param unit            the time unit for the {@code keepAliveTime} argument
     * @param workQueue       the queue to use for holding tasks before they are
     *                        executed.  This queue will hold only the {@code Runnable}
     *                        tasks submitted by the {@code execute} method.
     * @param threadFactory   the factory to use when the executor
     *                        creates a new thread
     * @throws IllegalArgumentException if one of the following holds:<br>
     *                                  {@code corePoolSize < 0}<br>
     *                                  {@code keepAliveTime < 0}<br>
     *                                  {@code maximumPoolSize <= 0}<br>
     *                                  {@code maximumPoolSize < corePoolSize}
     * @throws NullPointerException     if {@code workQueue}
     *                                  or {@code threadFactory} is null
     */
    public RequestExecutor(int corePoolSize, int maximumPoolSize, long keepAliveTime, TimeUnit unit, BlockingQueue<Runnable> workQueue, ThreadFactory threadFactory) {
        super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue, threadFactory);
    }

    /**
     * Creates a new {@code RequestExecutor} with the given initial
     * parameters and default thread factory.
     *
     * @param corePoolSize    the number of threads to keep in the pool, even
     *                        if they are idle, unless {@code allowCoreThreadTimeOut} is set
     * @param maximumPoolSize the maximum number of threads to allow in the
     *                        pool
     * @param keepAliveTime   when the number of threads is greater than
     *                        the core, this is the maximum time that excess idle threads
     *                        will wait for new tasks before terminating.
     * @param unit            the time unit for the {@code keepAliveTime} argument
     * @param workQueue       the queue to use for holding tasks before they are
     *                        executed.  This queue will hold only the {@code Runnable}
     *                        tasks submitted by the {@code execute} method.
     * @param handler         the handler to use when execution is blocked
     *                        because the thread bounds and queue capacities are reached
     * @throws IllegalArgumentException if one of the following holds:<br>
     *                                  {@code corePoolSize < 0}<br>
     *                                  {@code keepAliveTime < 0}<br>
     *                                  {@code maximumPoolSize <= 0}<br>
     *                                  {@code maximumPoolSize < corePoolSize}
     * @throws NullPointerException     if {@code workQueue}
     *                                  or {@code handler} is null
     */
    public RequestExecutor(int corePoolSize, int maximumPoolSize, long keepAliveTime, TimeUnit unit, BlockingQueue<Runnable> workQueue, RejectedExecutionHandler handler) {
        super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue, handler);
    }

    /**
     * Creates a new {@code RequestExecutor} with the given initial
     * parameters.
     *
     * @param corePoolSize    the number of threads to keep in the pool, even
     *                        if they are idle, unless {@code allowCoreThreadTimeOut} is set
     * @param maximumPoolSize the maximum number of threads to allow in the
     *                        pool
     * @param keepAliveTime   when the number of threads is greater than
     *                        the core, this is the maximum time that excess idle threads
     *                        will wait for new tasks before terminating.
     * @param unit            the time unit for the {@code keepAliveTime} argument
     * @param workQueue       the queue to use for holding tasks before they are
     *                        executed.  This queue will hold only the {@code Runnable}
     *                        tasks submitted by the {@code execute} method.
     * @param threadFactory   the factory to use when the executor
     *                        creates a new thread
     * @param handler         the handler to use when execution is blocked
     *                        because the thread bounds and queue capacities are reached
     * @throws IllegalArgumentException if one of the following holds:<br>
     *                                  {@code corePoolSize < 0}<br>
     *                                  {@code keepAliveTime < 0}<br>
     *                                  {@code maximumPoolSize <= 0}<br>
     *                                  {@code maximumPoolSize < corePoolSize}
     * @throws NullPointerException     if {@code workQueue}
     *                                  or {@code threadFactory} or {@code handler} is null
     */
    public RequestExecutor(int corePoolSize, int maximumPoolSize, long keepAliveTime, TimeUnit unit, BlockingQueue<Runnable> workQueue, ThreadFactory threadFactory, RejectedExecutionHandler handler) {
        super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue, threadFactory, handler);
    }
    //endregion

    /**
     * Submits a Runnable task for execution and returns a Future representing that task. The Future's get method will return null upon successful completion.
     * On the end of execution {@code listener} will be called.
     *
     * @param task     the task to submit
     * @param listener Listener for handling result of task execution
     * @param <T>      The result type returned by Api if success
     * @return a Future representing pending completion of the task
     */
    @NonNull
    <T> RequestFuture<T> submit(@NonNull Callable<T> task, @Nullable final RequestListener<T> listener) {
        RequestFutureTask<T> futureTask = new RequestFutureTask<>(task, listener);
        execute(futureTask);
        return futureTask;
    }

    /**
     * Submits a Runnable task for execution and returns a Future representing that task. The Future's get method will return null upon successful completion.
     *
     * @param task the task to submit
     * @param <T>  The result type returned by Api if success
     * @return a Future representing pending completion of the task
     */
    @NonNull
    @Override
    public <T> RequestFuture<T> submit(@NonNull Callable<T> task) {
        RequestFutureTask<T> futureTask = new RequestFutureTask<>(task);
        execute(futureTask);
        return futureTask;
    }

    //region Deprecated
    /**
     * {@inheritDoc}
     *
     * @deprecated This method doesn't handle execution exception, use {@link #submit(Callable, RequestListener)} or {@link #submit(Callable)} instead
     */
    @NonNull
    @Override
    @Deprecated
    public Future<?> submit(Runnable task) {
        throw new RuntimeException("Method not supported!");
    }

    /**
     * {@inheritDoc}
     *
     * @deprecated This method doesn't handle execution exception, use {@link #submit(Callable, RequestListener)} or {@link #submit(Callable)} instead
     */
    @NonNull
    @Override
    @Deprecated
    public <T> Future<T> submit(Runnable task, T result) {
        throw new RuntimeException("Method not supported!");
    }
    //endregion

}

==> rest-api-1.1.20-sources.jar: software/rsquared/restapi/RequestExecutor.java <==
package software.rsquared.restapi;

import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import software.rsquared.restapi.exceptions.RequestException;
import software.rsquared.restapi.listeners.ErrorCallback;
import software.rsquared.restapi.listeners.RequestListener;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.Callable;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.RejectedExecutionHandler;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * Wrapper for the {@link ThreadPoolExecutor} that allows to send signal to the {@link RequestListener} if execution finished and wraps exceptions to the {@link RequestException}
 *
 * @author Rafał Zajfert
 */
@SuppressWarnings("unused")
class RequestExecutor extends ThreadPoolExecutor {

    //region Constructors
    /**
     * Creates a new {@code RequestExecutor} default initial
     * parameters, thread factory and rejected execution handler.
     */
    RequestExecutor() {
        this(1, 0L);
    }

    /**
     * Creates a new {@code RequestExecutor} with the given initial
     * parameters and default thread factory and rejected execution handler.
     *
     * @param poolSize      the number of threads to keep in the pool, even
     *                      if they are idle, unless {@code allowCoreThreadTimeOut} is set
     * @param aliveDuration when the number of threads is greater than
     *                      the core, this is the maximum time (in milliseconds) that excess idle threads
     *                      will wait for new tasks before terminating.
     * @throws IllegalArgumentException if one of the following holds:<br>
     *                                  {@code corePoolSize < 0}<br>
     *                                  {@code keepAliveTime < 0}<br>
     */
    RequestExecutor(int poolSize, long aliveDuration) {
        super(poolSize, poolSize, aliveDuration, TimeUnit.MILLISECONDS, new LinkedBlockingQueue<Runnable>());
    }

    /**
     * Creates a new {@code RequestExecutor} with the given initial
     * parameters and default thread factory and rejected execution handler.
     *
     * @param corePoolSize    the number of threads to keep in the pool, even
     *                        if they are idle, unless {@code allowCoreThreadTimeOut} is set
     * @param maximumPoolSize the maximum number of threads to allow in the
     *                        pool
     * @param keepAliveTime   when the number of threads is greater than
     *                        the core, this is the maximum time that excess idle threads
     *                        will wait for new tasks before terminating.
     * @param unit            the time unit for the {@code keepAliveTime} argument
     * @param workQueue       the queue to use for holding tasks before they are
     *                        executed.  This queue will hold only the {@code Runnable}
     *                        tasks submitted by the {@code execute} method.
     * @throws IllegalArgumentException if one of the following holds:<br>
     *                                  {@code corePoolSize < 0}<br>
     *                                  {@code keepAliveTime < 0}<br>
     *                                  {@code maximumPoolSize <= 0}<br>
     *                                  {@code maximumPoolSize < corePoolSize}
     * @throws NullPointerException     if {@code workQueue} is null
     */
    public RequestExecutor(int corePoolSize, int maximumPoolSize, long keepAliveTime, TimeUnit unit, BlockingQueue<Runnable> workQueue) {
        super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue);
    }

    /**
     * Creates a new {@code RequestExecutor} with the given initial
     * parameters and default rejected execution handler.
     *
     * @param corePoolSize    the number of threads to keep in the pool, even
     *                        if they are idle, unless {@code allowCoreThreadTimeOut} is set
     * @param maximumPoolSize the maximum number of threads to allow in the
     *                        pool
     * @param keepAliveTime   when the number of threads is greater than
     *                        the core, this is the maximum time that excess idle threads
     *                        will wait for new tasks before terminating.
     * @param unit            the time unit for the {@code keepAliveTime} argument
     * @param workQueue       the queue to use for holding tasks before they are
     *                        executed.  This queue will hold only the {@code Runnable}
     *                        tasks submitted by the {@code execute} method.
     * @param threadFactory   the factory to use when the executor
     *                        creates a new thread
     * @throws IllegalArgumentException if one of the following holds:<br>
     *                                  {@code corePoolSize < 0}<br>
     *                                  {@code keepAliveTime < 0}<br>
     *                                  {@code maximumPoolSize <= 0}<br>
     *                                  {@code maximumPoolSize < corePoolSize}
     * @throws NullPointerException     if {@code workQueue}
     *                                  or {@code threadFactory} is null
     */
    public RequestExecutor(int corePoolSize, int maximumPoolSize, long keepAliveTime, TimeUnit unit, BlockingQueue<Runnable> workQueue, ThreadFactory threadFactory) {
        super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue, threadFactory);
    }

    /**
     * Creates a new {@code RequestExecutor} with the given initial
     * parameters and default thread factory.
     *
     * @param corePoolSize    the number of threads to keep in the pool, even
     *                        if they are idle, unless {@code allowCoreThreadTimeOut} is set
     * @param maximumPoolSize the maximum number of threads to allow in the
     *                        pool
     * @param keepAliveTime   when the number of threads is greater than
     *                        the core, this is the maximum time that excess idle threads
     *                        will wait for new tasks before terminating.
     * @param unit            the time unit for the {@code keepAliveTime} argument
     * @param workQueue       the queue to use for holding tasks before they are
     *                        executed.  This queue will hold only the {@code Runnable}
     *                        tasks submitted by the {@code execute} method.
     * @param handler         the handler to use when execution is blocked
     *                        because the thread bounds and queue capacities are reached
     * @throws IllegalArgumentException if one of the following holds:<br>
     *                                  {@code corePoolSize < 0}<br>
     *                                  {@code keepAliveTime < 0}<br>
     *                                  {@code maximumPoolSize <= 0}<br>
     *                                  {@code maximumPoolSize < corePoolSize}
     * @throws NullPointerException     if {@code workQueue}
     *                                  or {@code handler} is null
     */
    public RequestExecutor(int corePoolSize, int maximumPoolSize, long keepAliveTime, TimeUnit unit, BlockingQueue<Runnable> workQueue, RejectedExecutionHandler handler) {
        super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue, handler);
    }

    /**
     * Creates a new {@code RequestExecutor} with the given initial
     * parameters.
     *
     * @param corePoolSize    the number of threads to keep in the pool, even
     *                        if they are idle, unless {@code allowCoreThreadTimeOut} is set
     * @param maximumPoolSize the maximum number of threads to allow in the
     *                        pool
     * @param keepAliveTime   when the number of threads is greater than
     *                        the core, this is the maximum time that excess idle threads
     *                        will wait for new tasks before terminating.
     * @param unit            the time unit for the {@code keepAliveTime} argument
     * @param workQueue       the queue to use for holding tasks before they are
     *                        executed.  This queue will hold only the {@code Runnable}
     *                        tasks submitted by the {@code execute} method.
     * @param threadFactory   the factory to use when the executor
     *                        creates a new thread
     * @param handler         the handler to use when execution is blocked
     *                        because the thread bounds and queue capacities are reached
     * @throws IllegalArgumentException if one of the following holds:<br>
     *                                  {@code corePoolSize < 0}<br>
     *                                  {@code keepAliveTime < 0}<br>
     *                                  {@code maximumPoolSize <= 0}<br>
     *                                  {@code maximumPoolSize < corePoolSize}
     * @throws NullPointerException     if {@code workQueue}
     *                                  or {@code threadFactory} or {@code handler} is null
     */
    public RequestExecutor(int corePoolSize, int maximumPoolSize, long keepAliveTime, TimeUnit unit, BlockingQueue<Runnable> workQueue, ThreadFactory threadFactory, RejectedExecutionHandler handler) {
        super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue, threadFactory, handler);
    }
    //endregion

    /**
     * Submits a Runnable task for execution and returns a Future representing that task. The Future's get method will return null upon successful completion.
     * On the end of execution {@code listener} will be called.
     *
     * @param task     the task to submit
     * @param listener Listener for handling result of task execution
     * @param <T>      The result type returned by Api if success
     * @return a Future representing pending completion of the task
     */
    @NonNull
    <T> RequestFuture<T> submit(@NonNull Callable<T> task, @Nullable ErrorCallback errorCallback, @Nullable final RequestListener<T> listener) {
        RequestFutureTask<T> futureTask = new RequestFutureTask<>(task, errorCallback, listener);
        execute(futureTask);
        return futureTask;
    }

    /**
     * Submits a Runnable task for execution and returns a Future representing that task. The Future's get method will return null upon successful completion.
     *
     * @param task the task to submit
     * @param <T>  The result type returned by Api if success
     * @return a Future representing pending completion of the task
     */
    @NonNull
    @Override
    public <T> RequestFuture<T> submit(@NonNull Callable<T> task) {
        RequestFutureTask<T> futureTask = new RequestFutureTask<>(task, null);
        execute(futureTask);
        return futureTask;
    }

    //region Deprecated
    /**
     * {@inheritDoc}
     *
     * @deprecated This method doesn't handle execution exception, use {@link #submit(Callable, ErrorCallback, RequestListener)} or {@link #submit(Callable)} instead
     */
    @NonNull
    @Override
    @Deprecated
    public Future<?> submit(Runnable task) {
        throw new RuntimeException("Method not supported!");
    }

    /**
     * {@inheritDoc}
     *
     * @deprecated This method doesn't handle execution exception, use {@link #submit(Callable, ErrorCallback, RequestListener)} or {@link #submit(Callable)} instead
     */
    @NonNull
    @Override
    @Deprecated
    public <T> Future<T> submit(Runnable task, T result) {
        throw new RuntimeException("Method not supported!");
    }
    //endregion

}
